@@ -1,10 +1,10 @@
 /*
 
-    YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2020
+    YABASIC  ---  a simple Basic Interpreter
+    written by Marc Ihm 1995-2021
     more info at www.yabasic.de
 
-    io.c --- code for screen and file i/o
+    io.c -- code for screen and file i/o
 
     This file is part of yabasic and may be copied under the terms of
     MIT License which can be found in the file LICENSE.
@@ -106,7 +106,7 @@ create_print(char type)	/* create command 'print' */
 {
     struct command *cmd;
 
-    cmd = add_command(cPRINT, NULL, NULL);
+    cmd = add_command(cPRINT);
     cmd->pointer = my_malloc(sizeof (int));
     /* store type of print  */
     cmd->tag = type;
@@ -271,7 +271,7 @@ curinit(void)			/* initialize curses */
 #endif
 
 #ifdef UNIX
-    if (tcsetpgrp(STDIN_FILENO, getpid())) {
+    if (tcsetpgrp(STDIN_FILENO, getpgid(getpid()))) {
 	sprintf(string,"could not get control of terminal: %s",
                 my_strerror(errno));
         error (sERROR,string);
@@ -689,7 +689,7 @@ create_myopen(int num)		/* create command 'myopen' */
 {
     struct command *cmd;
 
-    cmd = add_command(cOPEN, NULL, NULL);
+    cmd = add_command(cOPEN);
     cmd->tag = num;
 }
 
@@ -941,7 +941,7 @@ create_pps(int type, int input)	/* create command push_stream or pop_stream */
 {
     struct command *cmd;
 
-    cmd = add_command(type, NULL, NULL);
+    cmd = add_command(type);
     cmd->args = input;
 }
 
@@ -1096,7 +1096,7 @@ create_myread(char type, int tileol)	/* create command 'read' */
 {
     struct command *cmd;
 
-    cmd = add_command(cREAD, NULL, NULL);
+    cmd = add_command(cREAD);
     cmd->args = tileol;		/* true, if read should go til eol */
     cmd->tag = type;		/* can be 'd' or 's' */
 }
@@ -1294,7 +1294,7 @@ create_onestring(char *str)	/* create command 'onestring' */
 {
     struct command *cmd;
 
-    cmd = add_command(cONESTRING, NULL, NULL);
+    cmd = add_command(cONESTRING);
     cmd->pointer = my_strdup(str);
 }
 
@@ -1340,7 +1340,7 @@ create_colour(int flag)	/* create command 'colour' */
 {
     struct command *c;
 
-    c = add_command(cCOLOUR, NULL, NULL);
+    c = add_command(cCOLOUR);
     c->args = flag;
 }
 
